using System;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.PostProcessing;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

namespace Feel.medicalpack
{
    public class MedicalPack : MVRScript
    {

        Syringe _syringe;
        Stethoscope _stethoscope;
        MedicineBottle _medicine_bottle;
        Pill _pill;

        private bool _is_init = false;
        private int _initLoops = 0;
        private readonly int _maxInitLoops = 1000;

        public override void Init()
        {
            try
            {
                if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("BreastPump - Not a usable environment." + containingAtom.type);
                    return;
                }

                _syringe = new Syringe(this);
                _stethoscope = new Stethoscope(this);
                _medicine_bottle = new MedicineBottle(this);
                _pill = new Pill(this);

                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>Medical Pack</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Control medical equipment.\n\n" +
                    "thank you for your support\n" +
                    "<b>https://www.patreon.com/feel_vam</b>\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 300.0f;

                var btn = CreateButton("Go To Support", true);
                btn.button.onClick.AddListener(() =>
                {
                    Application.OpenURL("https://www.patreon.com/feel_vam");
                });

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            if(_is_init)
            {
                if (_stethoscope != null) _stethoscope.SetUpdate();
            }
            else
            {
                if (_initLoops < _maxInitLoops)
                {
                    _initLoops++;
                    FirstInit();
                }
                else
                {
                    if (_initLoops == 1001)
                    {
                        SuperController.LogError("MedicalPack Load Failure " + _initLoops);
                        _initLoops += 1;
                    }
                }
            }
            
        }

        private void FirstInit()
        {
            bool isl = SuperController.singleton.isLoading;
            if (isl) return;

            Transform syringe = FeelUtil.getObjectRoot(containingAtom.reParentObject.transform, "syringe");
            Transform bic_syringe = FeelUtil.getObjectRoot(containingAtom.reParentObject.transform, "bic_syringe");
            Transform stethoscope = FeelUtil.getObjectRoot(containingAtom.reParentObject.transform, "Stethoscope");
            Transform medicine_bottle = FeelUtil.getObjectRoot(containingAtom.reParentObject.transform, "medicine bottle");
            Transform pill = FeelUtil.getObjectRoot(containingAtom.reParentObject.transform, "Pill");
            if (bic_syringe != null) syringe = null;

            //SuperController.LogMessage("1>> " + medicine_bottle);
            //SuperController.LogMessage("2>> " + bic_syringe);
            if (syringe != null || bic_syringe != null || stethoscope != null || medicine_bottle != null || pill != null)
            {
                if (syringe != null) _syringe.SetObj(syringe, "small");
                if (bic_syringe != null) _syringe.SetObj(bic_syringe, "bic");
                if (stethoscope != null) _stethoscope.SetObj(stethoscope);
                if (medicine_bottle != null) _medicine_bottle.SetObj(medicine_bottle);
                if (medicine_bottle == null && _pill != null) _pill.SetObj(pill);

                _is_init = true;
                _initLoops += 1001;

                SuperController.LogMessage("MedicalPack Init Complete. ");
            }

            
        }

        void FixedUpdate()
        {
            
        }


    }
	
}