using System;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.PostProcessing;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.Threading;

namespace Feel.medicalpack
{
    public class MedicineBottle
    {
        private MVRScript _root;
        private Transform _obj;
        private Transform _cap;
        private Transform _pills;

        private JSONStorableFloat _cap_pos;
        private JSONStorableBool _cap_show;
        private JSONStorableColor _pills_color;

        public MedicineBottle(MVRScript root)
        {
            _root = root;

            _cap_pos = new JSONStorableFloat("Cap pos", 0f, OnCapPosition, 0f, 1f);
            _root.RegisterFloat(_cap_pos);

            _cap_show = new JSONStorableBool("needle show", true, OnCapView);
            _root.RegisterBool(_cap_show);

            HSVColor pill_color = new HSVColor();
            pill_color.H = 0f;
            pill_color.S = 0.58f;
            pill_color.V = 0.76f;
            _pills_color = new JSONStorableColor("pills color", pill_color, OnPillColorChange);
            _root.RegisterColor(_pills_color);

        }

        public void SetObj(Transform obj)
        {
            _obj = obj;
            _cap = FeelUtil.getChildRoot(obj, "PillsCaseCover");
            _pills = FeelUtil.getChildRoot(obj, "Pills");

            ShowSyringeUI();
        }

        private void ShowSyringeUI()
        {
            _root.CreateSlider(_cap_pos, false);
            _root.CreateToggle(_cap_show);
            _root.CreateColorPicker(_pills_color);

            //SuperController.LogMessage(_pills_color.ToString());
            OnPillColorChange(_pills_color);
        }

        private void OnCapPosition(float value)
        {
            if (_cap.gameObject.activeSelf)
            {
                Vector3 n_v = _cap.transform.localPosition;
                n_v.y = value * 0.02f;
                SuperController.LogMessage("!!! " + n_v);
                _cap.transform.localPosition = n_v;
            }

        }

        private void OnCapView(bool is_show)
        {
            _cap.gameObject.SetActive(is_show);
        }

        private void OnPillColorChange(JSONStorableColor color)
        {
            if (_pills)
            {
                //SuperController.LogMessage(color.val.H + "-" + color.val.S + "-" + color.val.V);
                Renderer renderer = _pills.GetComponent<Renderer>();
                foreach(Material material in renderer.materials)
                {
                    Color c = Color.HSVToRGB(color.val.H, color.val.S, color.val.V);
                    material.SetColor("_Color", c);
                }
            }
            
        }

        public void SetUpdate()
        {
            
        }

    }

}