using System;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.PostProcessing;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.Threading;

namespace Feel.medicalpack
{
    public class Stethoscope
    {
        private MVRScript _root;
        private Transform _obj;
        private Transform _left_ear;
        private Transform _right_ear;
        private Transform _scope_bone;
        private Transform _scope_target;
        private Transform _head_bone;
        private Transform _ear_bone;
        private ForceReceiver _head_target;

        private Vector3 _l_a_b;
        private Vector3 _r_a_b;

        private JSONStorableFloat _stethoscope_angle;
        private JSONStorableStringChooser _obj_list;
        private UIDynamicPopup _obj_select_pop;
        private JSONStorableStringChooser _person_list;
        private UIDynamicPopup _person_select_pop;

        public Stethoscope(MVRScript root)
        {
            _root = root;

            _stethoscope_angle = new JSONStorableFloat("stethoscope angle", 0f, OnStethoscopeAngle, 0f, 1f);
            _root.RegisterFloat(_stethoscope_angle);

            _person_list = new JSONStorableStringChooser("Person Choice", null, null, "", OnSelectPerson);
            _person_list.storeType = JSONStorableParam.StoreType.Physical;
            _root.RegisterStringChooser(_person_list);


            _obj_list = new JSONStorableStringChooser("Object Choice", null, null, "", OnSelectOjbect);
            _obj_list.storeType = JSONStorableParam.StoreType.Physical;
            _root.RegisterStringChooser(_obj_list);
        }

        public void SetObj(Transform obj)
        {
            _obj = obj;
            _left_ear = FeelUtil.getChildRoot(_obj, "l_ear_0_bne");
            _right_ear = FeelUtil.getChildRoot(_obj, "r_ear_0_bne");
            _scope_bone = FeelUtil.getChildRoot(_obj, "c_hose_10_bne");
            _head_bone = FeelUtil.getChildRoot(_obj, "root_bne");
            _ear_bone = FeelUtil.getChildRoot(_obj, "c_cog_0_bne");

            _l_a_b = _left_ear.transform.localRotation.eulerAngles;
            _r_a_b = _right_ear.transform.localRotation.eulerAngles;

            ShowSyringeUI();
        }

        private void ShowSyringeUI()
        {
            _root.CreateSlider(_stethoscope_angle, false);

            _person_select_pop = _root.CreateScrollablePopup(_person_list, false);
            _person_select_pop.popupPanelHeight = 600f;
            _person_select_pop.popup.onOpenPopupHandlers += SyncPersonChoices;
            _person_select_pop.label = "Head Target:";

            _obj_select_pop = _root.CreateScrollablePopup(_obj_list, false);
            _obj_select_pop.popupPanelHeight = 600f;
            _obj_select_pop.popup.onOpenPopupHandlers += SyncPersonChoices;
            _obj_select_pop.label = "Scope Target:";

            UpdateAtomData();

            OnSelectOjbect(_obj_list.val);
            OnSelectPerson(_person_list.val);
        }

        private void OnStethoscopeAngle(float angle)
        {
            if(_left_ear != null)
            {
                Vector3 l_a = new Vector3(_l_a_b.x, _l_a_b.y, _l_a_b.z);
                Vector3 r_a = new Vector3(_r_a_b.x, _r_a_b.y, _r_a_b.z);
                //SuperController.LogMessage(l_a + "_" + r_a);

                float n_angle = (angle * 50f) - 20f;
                Quaternion l_a_q = Quaternion.Euler(_l_a_b.x, _l_a_b.y, _l_a_b.z + n_angle);
                Quaternion r_a_q = Quaternion.Euler(_r_a_b.x, _r_a_b.y, _r_a_b.z - n_angle);

                _left_ear.transform.localRotation = l_a_q;
                _right_ear.transform.localRotation = r_a_q;
            }
        }

        private void OnSelectOjbect(string target)
        {
            if(_scope_bone != null)
            {
                Rigidbody bone1_rigidbody = _scope_bone.GetComponent<Rigidbody>();
                if (SuperController.singleton.GetAtomByUid(target) != null)
                {
                    bone1_rigidbody.useGravity = false;
                    bone1_rigidbody.isKinematic = true;
                    _scope_target = SuperController.singleton.GetAtomByUid(target).mainController.transform;
                }
                else
                {
                    bone1_rigidbody.useGravity = true;
                    bone1_rigidbody.isKinematic = false;
                    _scope_target = null;
                }
            }

        }

        private void OnSelectPerson(string target)
        {
            if(SuperController.singleton.GetAtomByUid(target) != null && _head_bone != null)
            {
                Atom peason = SuperController.singleton.GetAtomByUid(target);

                _head_target = peason.forceReceivers.First(fc => fc.name == "head");
                _ear_bone.transform.localRotation = Quaternion.Euler(30f, 180f, 180f);
                _ear_bone.transform.localPosition = new Vector3(0f, -0.015f, 0.01f);

            }
        }

        public void SetUpdate()
        {
            if (_scope_target)
            {
                //SuperController.LogMessage("SetUpdate");
                _scope_bone.transform.position = _scope_target.transform.position;
                _scope_bone.transform.rotation = _scope_target.transform.rotation;
            }
            if(_head_target)
            {
                Vector3 h_v = _head_target.transform.rotation.eulerAngles;
                Quaternion n_hv = Quaternion.Euler(h_v.x - 8f, h_v.y + 70f, h_v.z + 45f);
                _head_bone.transform.position = _head_target.transform.position;
                _head_bone.transform.rotation = _head_target.transform.rotation;
                //_head_bone.transform.Rotate(new Vector3(0f, 90f, 0f));

            }
        }

        private void UpdateAtomData()
        {
            List<string> atomChoices = new List<string>();
            List<string> objChoices = new List<string>();
            Atom atom;

            foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
            {
                atom = SuperController.singleton.GetAtomByUid(atomUID);
                JSONStorable geometry = atom.GetStorableByID("geometry");
                if (geometry != null)
                {
                    atomChoices.Add(atomUID);
                }
                else
                {
                    objChoices.Add(atomUID);
                }
                
            }

            _person_list.choices = atomChoices;
            _obj_list.choices = objChoices;
        }

        private void SyncPersonChoices()
        {

        }

    }

}