using System;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.PostProcessing;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

namespace Feel.medicalpack
{
    public class Syringe
    {
        private MVRScript _root;
        private Transform _obj;
        private Transform _needle;
        private Transform _barrel;
        private Transform _adapter;
        private Transform _plunger;
        private Transform _plunger_rubber;
        private Transform _liquid_bone;
        private Transform _liquid;
        private Material _liquid_material;

        private JSONStorableFloat _syringe_value;
        private JSONStorableBool _liquid_show;
        private JSONStorableColor _liquid_color;
        private JSONStorableFloat _emission_value;
        private JSONStorableFloat _barrel_alpha;
        private JSONStorableBool _needle_show;

        private string _type;

        public Syringe(MVRScript root)
        {
            _root = root;

            _syringe_value = new JSONStorableFloat("syringe value", 0f, OnSyringeValue, 0f, 1f);
            _root.RegisterFloat(_syringe_value);

            _liquid_show = new JSONStorableBool("liquid show", true, OnLiquidShow);
            _root.RegisterBool(_liquid_show);

            HSVColor default_color = new HSVColor();
            _liquid_color = new JSONStorableColor("liquid color", default_color, OnLiquidColor);
            _root.RegisterColor(_liquid_color);

            _emission_value = new JSONStorableFloat("emission value", 0f, OnEmissionValue, 0f, 1.5f);
            _root.RegisterFloat(_emission_value);

            _barrel_alpha = new JSONStorableFloat("barrel alpha", 1f, OnBarrelAlpha, 0f, 1f);
            _root.RegisterFloat(_barrel_alpha);

            _needle_show = new JSONStorableBool("needle show", true, OnNeedleShow);
            _root.RegisterBool(_needle_show);
        }

        public void SetObj(Transform obj, string type)
        {
            _type = type;
            _needle = FeelUtil.getChildRoot(obj, "needle");
            _adapter = FeelUtil.getChildRoot(obj, "adapter");
            _barrel = FeelUtil.getChildRoot(obj, "barrel");
            _plunger = FeelUtil.getChildRoot(obj, "plunger");
            if(_type == "small")
                _liquid_bone = FeelUtil.getChildRoot(obj, "Bone");
            else
                _liquid_bone = FeelUtil.getChildRoot(obj, "Bone.001");

            _liquid = FeelUtil.getChildRoot(obj, "Liquid");
            _plunger_rubber = FeelUtil.getChildRoot(_plunger, "plunger_rubber");

            Renderer renderer = _liquid.GetComponent<Renderer>();
            _liquid_material = renderer.materials[0];

            OnSyringeValue(_syringe_value.val);
            OnLiquidShow(_liquid_show.val);
            OnLiquidColor(_liquid_color);
            OnEmissionValue(_emission_value.val);
            OnBarrelAlpha(_barrel_alpha.val);
            OnNeedleShow(_needle_show.val);

            ShowSyringeUI();
        }

        private void ShowSyringeUI()
        {
            _root.CreateSlider(_syringe_value, false);
            _root.CreateToggle(_liquid_show);
            _root.CreateColorPicker(_liquid_color, false);
            _root.CreateSlider(_emission_value, false);
            _root.CreateSlider(_barrel_alpha, false);
            _root.CreateToggle(_needle_show);
        }

        private void OnSyringeValue(float value)
        {
            if(_plunger != null)
            {
                if(_type == "small")
                    _plunger.localPosition = new Vector3(0f, -0.9f - (value * 1.1f), 0f);
                else
                    _plunger.localPosition = new Vector3(0f, -2.23f - (value * 7.7f), 0f);
                _liquid_bone.localScale = new Vector3(1f, value, 1f);
            }
            
        }

        private void OnLiquidShow(bool show)
        {
            //SuperController.LogMessage("OnLiquidShow " + show);
            if(_liquid != null) _liquid.gameObject.SetActive(show);
        }

        private void OnNeedleShow(bool show)
        {
            //SuperController.LogMessage("OnLiquidShow " + show);
            if (_needle != null) _needle.gameObject.SetActive(show);
            if (_adapter != null) _adapter.gameObject.SetActive(show);
        }

        private void OnLiquidColor(JSONStorableColor color)
        {
            if(_liquid_material != null)
            {
                Color c = Color.HSVToRGB(color.valNoCallback.H, color.valNoCallback.S, color.valNoCallback.V);
                _liquid_material.SetColor("_Color", c);
            }
            
        }

        private void OnEmissionValue(float value)
        {
            if (_liquid_material != null)
            {
                _liquid_material.SetColor("_EmissionColor", Color.white * _emission_value.val);
            }

        }

        private void OnBarrelAlpha(float value)
        {
            if (_liquid_material != null)
            {
                Color c = _barrel.GetComponent<Renderer>().material.GetColor("_Color");
                c.a = _barrel_alpha.val;

                _barrel.GetComponent<Renderer>().material.SetColor("_Color", c);
                _plunger.GetComponent<Renderer>().material.SetColor("_Color", c);
            }

        }

    }

}